# coding=utf-8
from django.shortcuts import get_object_or_404, redirect

from notifications import get_today_midnight_date, get_filter_locations, get_calendar_full_appointments, \
    get_unfinished_appointments
from . import wrap_response
from ..forms import AppointmentDetailForm, AppointmentAddForm, AppointmentEditForm, SubjectEditForm
from ..models import Appointment


def appointments(request):
    approaching_list = Appointment.objects.filter(
        datetime_when__gt=get_today_midnight_date(),
        location__in=get_filter_locations(request.user),
        status=Appointment.APPOINTMENT_STATUS_SCHEDULED
    ).order_by('datetime_when')

    full_list = get_calendar_full_appointments(request.user)

    context = {
        'approaching_list': approaching_list,
        'full_list': full_list
    }

    return wrap_response(request, "appointments/index.html", context)


def unfinished_appointments(request):
    appointments = get_unfinished_appointments(request.user)
    context = {
        'appointment_list': appointments,
    }

    return wrap_response(request, "appointments/list.html", context)


def appointment_details(request, id):
    the_appointment = get_object_or_404(Appointment, id=id)
    form = AppointmentDetailForm(instance=the_appointment)
    return wrap_response(request, 'appointments/details.html', {'form': form})


def appointment_add(request, visit_id=None):
    full_list = get_calendar_full_appointments(request.user)
    if request.method == 'POST':
        form = AppointmentAddForm(request.POST, request.FILES, user=request.user)
        if form.is_valid():
            form.instance.visit_id = visit_id
            form.save()
            if visit_id is None:
                return redirect('web.views.appointments')
            else:
                return redirect('web.views.visit_details', id=visit_id)
    else:
        form = AppointmentAddForm(user=request.user)

    return wrap_response(request, 'appointments/add.html',
                         {'form': form, 'visitID': visit_id, 'full_appointment_list': full_list})


def appointment_edit(request, id):
    the_appointment = get_object_or_404(Appointment, id=id)
    subject_form = None

    if request.method == 'POST':
        appointment_form = AppointmentEditForm(request.POST,
                                               request.FILES,
                                               instance=the_appointment,
                                               user=request.user,
                                               prefix="appointment")

        is_valid_form = True
        if the_appointment.visit is not None:
            subject_form = SubjectEditForm(request.POST, instance=the_appointment.visit.subject, prefix="subject")
            if not subject_form.is_valid():
                is_valid_form = False

        if not appointment_form.is_valid():
            is_valid_form = False

        if is_valid_form:
            appointment_form.save()
            if subject_form is not None:
                subject_form.save()
            the_appointment = get_object_or_404(Appointment, id=id)
            if (the_appointment.status != Appointment.APPOINTMENT_STATUS_SCHEDULED) and (
                the_appointment.visit is not None):
                return redirect('web.views.visit_details', id=the_appointment.visit.id)
            else:
                return redirect('web.views.appointments')
    else:
        appointment_form = AppointmentEditForm(instance=the_appointment, user=request.user, prefix="appointment")
        if the_appointment.visit is not None:
            subject_form = SubjectEditForm(instance=the_appointment.visit.subject, prefix="subject")

    return wrap_response(request, 'appointments/edit.html', {
        'form': appointment_form,
        'subject_form': subject_form,
        'id': id,
        'appointment': the_appointment
    })
